library(logmult)

load("weights NUTS2b 2014-2016.RData")

load("tab NUTS2b eseg 2014-2016.RData")
tab.eseg <- tab[1:7,1:7,]
tabw.eseg <- tabw[1:7,1:7,]
tab.eseg <- as.table(tab[1:7, 1:7, !dimnames(tab)[[3]] %in% c("ES7", "FRA", "PT20", "PT30")])
tabw.eseg <- as.table(tabw[1:7, 1:7, !dimnames(tabw)[[3]] %in% c("ES7", "FRA", "PT20", "PT30")])

load("tab NUTS2b educ2 2014-2016.RData")
tab.educ2 <- tab
tabw.educ2 <- tabw
tab.educ2 <- as.table(tab[,, !dimnames(tab)[[3]] %in% c("ES7", "FRA", "PT20", "PT30")])
tabw.educ2 <- as.table(tabw[,, !dimnames(tabw)[[3]] %in% c("ES7", "FRA", "PT20", "PT30")])

# Normalize weighted tables so that they sum to the actual sample size for each region
tabw2.eseg <- sweep(tabw.eseg, 3, margin.table(tab.eseg, 3)/margin.table(tabw.eseg, 3), "*")
tabw2.educ2 <- sweep(tabw.educ2, 3, margin.table(tab.educ2, 3)/margin.table(tabw.educ2, 3), "*")
tabw2.educ2.orig <- tabw2.educ2
# A few empty cells in this region prevent fitting the UNIDIFF + diagonals model
tabw2.educ2[,,"SK01"] <- tabw2.educ2[,,"SK01"] + 0.5
tabw2.educ2[,,"SK03"] <- tabw2.educ2[,,"SK03"] + 0.5

# Model fitting may have to be restarted a couple of times if it fails to converge
u.eseg <- unidiff(tabw2.eseg, eliminate=NA, checkEstimability=FALSE)
u.educ2 <- unidiff(tabw2.educ2, eliminate=NA, checkEstimability=FALSE)

ud.eseg  <- unidiff(tabw2.eseg, diagonal="excluded", eliminate=NA, checkEstimability=FALSE)
ud.educ2  <- unidiff(tabw2.educ2, diagonal="excluded", eliminate=NA, checkEstimability=FALSE)

homo.df <- data.frame(eseg=iac(tabw2.eseg + 0.5),
                      eseg.b=iac(tabw2.eseg + 0.5, shrink=TRUE),
                      eseg.u=iac(fitted(u.eseg)),
                      eseg.ud=iac(fitted(ud.eseg)),
                      educ2=iac(tabw2.educ2.orig + 0.5),
                      educ2.b=iac(tabw2.educ2.orig + 0.5, shrink=TRUE),
                      educ2.u=iac(fitted(u.educ2)),
                      educ2.ud=iac(fitted(ud.educ2)))

homo.df$w <- c(wts[rownames(homo.df)])


indep.educ2 <- gnm(Freq ~ educ2H*NUTS2b + educ2F*NUTS2b,
                   data=tabw2.educ2, family=poisson, eliminate=educ2H:NUTS2b)
indep.eseg <- gnm(Freq ~ esegH*NUTS2b + esegF*NUTS2b,
                  data=tabw2.eseg, family=poisson, eliminate=esegH:NUTS2b)

stable.educ2 <- gnm(Freq ~ educ2H*NUTS2b + educ2F*NUTS2b + educ2H:educ2F,
                    data=tabw2.educ2, family=poisson, eliminate=educ2H:NUTS2b)
stable.eseg <- gnm(Freq ~ esegH*NUTS2b + esegF*NUTS2b + esegH:esegF,
                   data=tabw2.eseg, family=poisson, eliminate=esegH:NUTS2b)

models.summary <- function(models) {
    stopifnot(all(sapply(models, function(model) all(model$data == models[[1]]$data, na.rm=TRUE))))

    df <- sapply(models, function(model) if(!is.null(model)) model$df.residual else NA)
    deviances <- sapply(models, function(model) if(!is.null(model)) round(model$deviance) else NA)
    errors <- sapply(models, function(model) {
        if(!is.null(model)) round(sum(abs(fitted(model)-model$data), na.rm=TRUE)/sum(abs(fitted(model)), na.rm=TRUE)*100/2, digits=2)
        else NA })
    bics <- sapply(models, function(model) if(!is.null(model)) model$dev - log(sum(model$data, na.rm=TRUE)) * model$df.residual)
    aics <- sapply(models, function(model) if(!is.null(model)) model$dev - 2 * model$df.residual)

    for(i in 1:length(models)) if(!isTRUE(models[[i]]$converged)) warning("Model ", names(models)[i], " did not converge!")
    data.frame(row.names=names(models), D.F.=df, Deviance=deviances, "Δ (%)"=errors, BIC=round(bics), AIC=round(aics), check.names=FALSE)
}

# Table 1: Fit statistics for log-linear and log-multiplicative models
models.summary(list(Independence=indep.eseg, Stability=stable.eseg,
                    UNIDIFF=u.eseg, "UNIDIFF + diagonal"=ud.eseg))
models.summary(list(Independence=indep.educ2, Stability=stable.educ2,
                    UNIDIFF=u.educ2, "UNIDIFF + diagonal"=ud.educ2))


# Table 2: Correlations Between the Indices Estimated via Four Different Methods
cov.wt(homo.df, homo.df$w, cor=T)$cor

# Table 3: Average, Dispersion and Range of the Indices Estimated via Four Different Methods
cov.wt(homo.df, homo.df$w)$center
c(min(homo.df$eseg), min(homo.df$eseg.b), min(homo.df$eseg.u), min(homo.df$eseg.ud))
c(max(homo.df$eseg), max(homo.df$eseg.b), max(homo.df$eseg.u), max(homo.df$eseg.ud))
c(min(homo.df$educ2), min(homo.df$educ2.b), min(homo.df$educ2.u), min(homo.df$educ2.ud))
c(max(homo.df$educ2), max(homo.df$educ2.b), max(homo.df$educ2.u), max(homo.df$educ2.ud))

source("metro.R")

rev <- read.csv2("Disposable income PPS NUTS2.csv", comment.char="#", stringsAsFactors=FALSE)
rev$GEO <- sub("EL", "GR", rev$GEO)
homo.df$NUTS2 <- fix.nuts2(rownames(homo.df))
homo.df$rev06 <- c(scale(rev[match(homo.df$NUTS2, rev$GEO), "X2006"]))
homo.metros <- merge(homo.df, subset(metros, !duplicated(NUTS2), by="NUTS2"), all.x=TRUE)
homo.metros$Type.of.City[is.na(homo.metros$Type.of.City)] <- "Other"
homo.metros$Type.of.City <- relevel(factor(homo.metros$Type.of.City), "Other")
homo.metros$COUNTRY <- substr(homo.metros$NUTS2, 1, 2)
rownames(homo.metros) <- homo.metros$NUTS2

homo.metros <- subset(homo.metros, complete.cases(homo.metros[c("COUNTRY", "Type.of.City", "rev06")]))

lm.eseg <- lm(log(eseg) ~ COUNTRY + rev06 + I(rev06^2) + Type.of.City, homo.metros, weights=w/mean(w))
lm.eseg.b <- lm(log(eseg.b) ~ COUNTRY + rev06 + I(rev06^2) + Type.of.City, homo.metros, weights=w/mean(w))
lm.eseg.u <- lm(log(eseg.u) ~ COUNTRY + rev06 + I(rev06^2) + Type.of.City, homo.metros, weights=w/mean(w))
lm.eseg.ud <- lm(log(eseg.ud) ~ COUNTRY + rev06 + I(rev06^2) + Type.of.City, homo.metros, weights=w/mean(w))

lm.educ2 <- lm(log(educ2) ~ COUNTRY + rev06 + I(rev06^2) + Type.of.City, homo.metros, weights=w/mean(w))
lm.educ2.b <- lm(log(educ2.b) ~ COUNTRY + rev06 + I(rev06^2) + Type.of.City, homo.metros, weights=w/mean(w))
lm.educ2.u <- lm(log(educ2.u) ~ COUNTRY + rev06 + I(rev06^2) + Type.of.City, homo.metros, weights=w/mean(w))
lm.educ2.ud <- lm(log(educ2.ud) ~ COUNTRY + rev06 + I(rev06^2) + Type.of.City, homo.metros, weights=w/mean(w))

# Table 4: Linear Regression Results for Relative Socioeconomic Homogamy
# R² within
R2w.eseg <- sapply(list(lm.eseg, lm.eseg.b, lm.eseg.u, lm.eseg.ud),
                   function(x) sum(deviance(x))/sum(deviance(update(x, formula=~ 1))))
R2w.educ2 <- sapply(list(lm.educ2, lm.educ2.b, lm.educ2.u, lm.educ2.ud), 
                    function(x) sum(deviance(x))/sum(deviance(update(x, formula=~ 1))))

library(stargazer)
mods <- list(lm.eseg, lm.eseg.b, lm.eseg.u, lm.eseg.ud)
boot.mod <- function(data, ind, mod) coef(update(mod, data=data[ind,]))[names(coef(mod))]
ci <- lapply(mods, function(x) {
                 b <- boot::boot(homo.metros, boot.mod, 5000, mod=x, strata=factor(substr(rownames(homo.metros), 1, 2)))
                 t(exp(sapply(seq(length(coef(x))), function(i) boot::boot.ci(b, .95, "norm", index=i)$norm)[2:3,]))
             })
stargazer(mods, type="text", keep.stat=c("n", "rsq", "adj.rsq"), report="vcs",
          decimal.mark=",", digits=2, omit.table.layout = "n",
          ci.custom=ci,
          apply.coef=exp,
          covariate.labels=c("Disposable income", "Disposable income²",
                             "Capital", "Second-tier metropolis"),
          dep.var.caption="",
          model.numbers=F, column.labels=c("Non-parametric", "Shrinkage", "UNIDIFF", "UNIDIFF + Diagonal"),
          omit="COUNTRY", omit.yes.no=c("×", ""), omit.labels = "Country fixed effects")
