library(dplyr)

## Transformation des .csv en .RData
reload <- FALSE

if(reload) {
    files1 <- list.files("Bases/Yearly Files 2016", pattern=".*201(1|2|3|4|5|6).*\\.csv", full.names=TRUE)
    # Pays scandinaves et Luxembourg (post-2015) qui fournissent un fichier ménages distinct    
    files2 <- list.files("Bases/Yearly Special Files 2016", pattern=".*201(1|2|3|4|5|6).*\\.csv", full.names=TRUE)

    for(file in c(files1, files2)) {
        cat(file, "\n")
        
        ind <- read.csv(file, stringsAsFactors=FALSE)

        # Pays, région NUTS-2, année (référence/enquête), pondération annuelle, sexe, âge,
        ind <- subset(ind, select=c(COUNTRY, REGION, REFYEAR, YEAR, COEFF, SEX, AGE,
                                    # Mois de référence, trimestre
                                    REM, QUARTER,
                                    # Id et type du foyer, foyer privé, id de la personne dans le foyer, du conjoint dans le foyer, lien avec la personne de réf., statut marital,
                                    QHHNUM, HHTYPE, HHPRIV, HHSEQNUM, HHSPOU, HHLINK, MARSTAT,
                                    # Statut d'activité, statut pro, statut principal, ISCO
                                    ILOSTAT, STAPRO, STAPROPR, MAINSTAT, ISCO1D, ISCO3D, ISCOPR1D, ISCOPR3D,
                                    # Secteur d'activité, supervision, taille de l'établissement, mois depuis dernier emploi
                                    NACE1D, NACEPR1D, SUPVISOR, SIZEFIRM, LEAVTIME,
                                    # Niveau (détaillé, agrégé) et domaine d'éducation validé
                                    HAT11LEV, HATLEV1D, HATFIELD,
                                    # Degré d'urbanisation
                                    DEGURBA))

        # Ici on garde tout le monde, les actifs occupés (ilostat=1) et les chômeurs (ilostat=2),
        # les inactifs dont les retraités (ilostat=3), le service militaire (ilostat=4).
        # On exclut les moins de 15 ans, càd ilostat=9, et les individus ne vivant pas en ménage ordinaire (HHPRIV!="1")
        # puisque la plupart des pays n'interrogent pas les autres
        ind <- subset(ind, ILOSTAT %in% 1:4 & HHPRIV == "1" & COEFF > 0)

        save(ind, file=sub(".csv", ".RData", file))

        rm(ind)
        gc()
    }
}

## Chargement des .RData et stockage des tables séparées dans la base SQL

files1 <- list.files("Bases/Yearly Files 2016", pattern=".*201(1|2|3|4|5|6).*\\.RData", full.names=TRUE)
# Malte ne fournit que l'ISCO à un chiffre (de même que la Bulgarie dans certains cas)
# Les autres ne permettent pas de reconstruire les couples (dont le Luxembourg à partir de 2015)
files1 <- files1[!substr(basename(files1), 1, 2) %in% c("DK", "FI", "MT", "CH", "IS", "SE")]
files1 <- files1[!substr(basename(files1), 1, 6) %in% c("LU2015", "LU2016")]

# Pays scandinaves qui fournissent un fichier ménages distinct
files2 <- list.files("Bases/Yearly Special Files 2016/", pattern=".*201(1|2|3|4|5|6).*\\.RData", full.names=TRUE)
# En Finlande, ISCO2D n'est définie que pour au maximum une personne par ménage même dans ce fichier
# Au Danemark, c'est l'éducation qui est très souvent inconnue
files2 <- files2[!substr(basename(files2), 1, 2) %in% c("DK", "FI")]

files <- c(files1, files2)

#lfs <- src_sqlite("Bases/lfs.sqlite3", create=TRUE)
lfs <- MonetDBLite::src_monetdblite("Bases/lfs.monetdb/", create=TRUE)

indl <- list()

for(file in files) {
    cat(file, "\n")

    load(file)

    ind <- mutate(ind,
        QHHNUM = as.integer(factor(QHHNUM)),

        # Pour éviter le type logical lorsque la variable est vide (Norvège avant 2013)
        HHLINK = as.integer(HHLINK),
        MAINSTAT = as.integer(MAINSTAT),
    
        COUNTRY2 = case_when(COUNTRY == "DE" & substr(REGION, 0, 1) %in% c("3", "4", "8", "D", "E", "G") ~ "DE-E",
                             COUNTRY == "DE" & substr(REGION, 0, 1) %in% c("1", "2", "5", "6", "7", "9", "A", "B", "C", "F") ~ "DE-W",
                             TRUE ~ COUNTRY),
        REGION = as.character(REGION),
        REGION2 = if_else(REGION %in% 0:9, paste0("0", REGION), as.character(REGION)),
        NUTS1 = paste0(COUNTRY, substr(REGION2, 1, 1)),
                           # Bavière, Baden-Württemberg
        NUTS1b = case_when(NUTS1 %in% c("DE1", "DE2") ~ "DE-S",
                           # Ex-RDA + Berlin : Berlin, Brandebourg et Mecklenbourg-Poméranie occidentale, Saxe, Saxe-Anhalt et Thuringe
                           NUTS1 %in% c("DE3", "DE4", "DE8", "DED", "DEE", "DEG") ~ "DE-E",
                           # Brême, Hambourg, Hessen, Saxe inférieure, Rhénanie du Nord-Westphalie, Rhénanie-Palatinat, Sarre et Schleswig-Holstein
                           NUTS1 %in% c("DE5", "DE6", "DE7", "DE9", "DEA", "DEB", "DEC", "DEF") ~ "DE-NW",
                           
                           # Nord-Est, Nord-Ouest de l'Angleterre, et Yorkshire and the Humber
                           NUTS1 %in% c("UKC", "UKD", "UKE") ~ "UKCDE",
                           # Midlands de l'Est et de l'Ouest
                           NUTS1 %in% c("UKF", "UKG") ~ "UKFG",
                           # Est, Sud Est et Sud Ouest de l'Angleterre
                           NUTS1 %in% c("UKH", "UKJ", "UKK") ~ "UKHJK",
                           # Pays de Galles et Irlande du Nord (faute de mieux...)
                           NUTS1 %in% c("UKL", "UKN") ~ "UKLN",
                           TRUE ~ NUTS1),

        NUTS2 = paste0(COUNTRY, REGION2),
        # Pays sans NUTS2 ou avec trop peu de monde par NUTS2
        NUTS2b = if_else(COUNTRY %in% c("AT", "BG", "CY", "DE", "EE", "ES", "FR", "GR",
                                        "LT", "LU", "LV", "NL", "NO", "UK") |
                         NUTS1 == "SE3",
                         NUTS1, NUTS2),
        # Fusionner Prague avec la région qui l'entoure (trop petite sinon),
        # qui est elle-même déjà considérée comme appartenant à son aire urbaine
        NUTS2b = if_else(NUTS2 == "CZ01", "CZ02", NUTS2b),
    
        SEX = recode_factor(SEX, "1"="H", "2"="F"),
        AGE5 = cut(AGE, c(0, 25, 35, 45, 55, 65, 75, 1000),
                   labels=c("0-24", "25-34", "35-44", "45-54", "55-64", "65-74", "75-100")),
        AGE10 = cut(AGE, c(0, 20, 30, 40, 50, 60, 70, 80, 1000),
                    labels=c("0-19", "20-29", "30-39", "40-49", "50-59", "60-69", "70-79", "80-100")),
    
        ISCO1D = as.numeric(as.character(na_if(na_if(ISCO1D, "999"), "")))/100,
        ISCOPR1D = as.numeric(as.character(na_if(na_if(ISCOPR1D, "999"), "")))/100,
    
        # SI et PL contiennent "0 0"
        ISCO3D = as.numeric(recode(as.character(na_if(na_if(ISCO3D, "999"), "")),  "0 0"="000")),
        ISCOPR3D = as.numeric(recode(as.character(na_if(na_if(ISCOPR3D, "999"), "")), "0 0"="000")),
    
        ISCO2D = floor(as.numeric(as.character(ISCO3D))/10),
        ISCOPR2D = floor(as.numeric(as.character(ISCOPR3D))/10),
        ISCO2D = as.numeric(if_else(ISCO2D < 10, paste0("0", ISCO2D), as.character(ISCO2D))),
        ISCOPR2D = as.numeric(if_else(ISCOPR2D < 10, paste0("0", ISCOPR2D), as.character(ISCOPR2D))),

        # On définit les variables de base pour les chômeurs et les inactifs quand c'est possible
        ISCO1D = if_else(ILOSTAT %in% c("2", "3", "4"), ISCOPR1D, ISCO1D),
        ISCO2D = if_else(ILOSTAT %in% c("2", "3", "4"), ISCOPR2D, ISCO2D),
        ISCO3D = if_else(ILOSTAT %in% c("2", "3", "4"), ISCOPR3D, ISCO3D),
        STAPRO = if_else(ILOSTAT %in% c("2", "3", "4") & STAPROPR %in% c("0", "3", "4"),
                         STAPROPR, STAPRO),
        NACE1D = if_else(ILOSTAT %in% c("2", "3", "4") & NACEPR1D %in% LETTERS[1:21],
                         as.character(NACEPR1D), as.character(NACE1D)),

                              # Les militaires sont trop peu nombreux, et classés de manière plus ou moins précise selon les pays
        ISCO2Dbis = case_when(ISCO2D %in% c(0, 01, 02, 03) ~ 0,
                              # La Lituanie est la seule à avoir des 60, peu de pays ont des 63, et de toute façon ces catégories sont très peu nombreuses
                              ISCO2D %in% c(60, 61, 62, 63) ~ 60,
                              # Les vendeurs de rue sont très peu nombreux à part en Italie : 96 devient "autres professions élémentaires"
                              ISCO2D == 95 ~ 96,
                              TRUE ~ ISCO2D),

        eseg = case_when(ISCO1D == "1" | ISCO2D == "01"                         ~ "Managers",
                         ISCO1D == "2"                                          ~ "Professionals",
                         (ISCO1D == "3" | ISCO2D == "02") & STAPRO == "3"       ~ "Associate professionals",
                         ISCO1D %in% c("3", "4", "5", "6", "7", "8", "9") &
                             STAPRO %in% c("0", "4")                            ~ "Small entrepreneurs",
                         (ISCO1D == "4" | ISCO2D %in% c("03", "00", "53", "54", "50")) &
                             STAPRO == "3"                                      ~ "Skilled white-collar",
                         ISCO1D %in% c("7", "8") & STAPRO == "3"                ~ "Skilled blue-collar",
                         (ISCO1D %in% c("6", "9") | ISCO2D %in% c("51", "52")) &
                             STAPRO == "3"                                      ~ "Less skilled",
                         # Catégories par défaut pour les chômeurs et inactifs non classés
                         ILOSTAT == "2" | (ILOSTAT %in% c("3", "4") & AGE < 65) ~ "Inactive",
                         ILOSTAT == "3" & (MAINSTAT == "4" | AGE >= 65)         ~ "Retired"),

        eseg = factor(eseg, levels=c("Managers", "Professionals", "Associate professionals", "Small entrepreneurs", 
                                     "Skilled white-collar", "Skilled blue-collar", "Less skilled",
                                     "Retired", "Inactive")),

        # STAPRO 0, 4 : indépendant, aide familial
        # STAPRO 3 : salarié
        eseg2 = case_when(ISCO2D %in% c("11", "12", "13") & STAPRO %in% c("0", "4") ~ "11",
                          ISCO2D == "14" & STAPRO %in% c("0", "4") ~ "12",
                          ISCO2D %in% c("01", "11", "12", "13") & STAPRO == "3" ~ "13",
                          ISCO2D == "14" & STAPRO == "3" ~ "14",
                          
                          ISCO2D %in% c("21", "25") ~ "21",
                          ISCO2D == "22" ~ "22",
                          ISCO2D == "24" ~ "23",
                          ISCO2D == "26" ~ "24",
                          ISCO2D == "23" ~ "25",
                          
                          ISCO2D %in% c("31", "35") & STAPRO == "3" ~ "31",
                          ISCO2D == "32" & STAPRO == "3" ~ "32",
                          ISCO2D == "33" & STAPRO == "3" ~ "33",
                          ISCO2D == "34" & STAPRO == "3" ~ "34",
                          ISCO2D == "02" & STAPRO == "3" ~ "35",
                          
                          # Utiliser ISCO2D pour avoir le même champ (et pas plus de gens)
                          # que pour les autres catégories
                          substr(ISCO2D, 0, 1) == "6" & STAPRO %in% c("0", "4") ~ "41",
                          substr(ISCO2D, 0, 1) %in% c("3", "4", "5") & STAPRO %in% c("0", "4") ~ "42",
                          substr(ISCO2D, 0, 1) %in% c("7", "8", "9") & STAPRO %in% c("0", "4") ~ "43",
                          
                          ISCO2D %in% c("41", "43", "44") & STAPRO == "3" ~ "51",
                          ISCO2D == "42" & STAPRO == "3" ~ "52",
                          ISCO2D == "53" & STAPRO == "3" ~ "53",
                          ISCO2D %in% c("03", "54") & STAPRO == "3" ~ "54",
                          
                          ISCO2D == "71" & STAPRO == "3" ~ "61",
                          ISCO2D == "75" & STAPRO == "3" ~ "62",
                          ISCO2D %in% c("72", "73", "74") & STAPRO == "3" ~ "63",
                          ISCO2D %in% c("81", "82") & STAPRO == "3" ~ "64",
                          ISCO2D == "83" & STAPRO == "3" ~ "65",
                          
                          ISCO2D %in% c("51", "52") & STAPRO == "3" ~ "71",
                          ISCO2D %in% c("92", "93", "94", "96") & STAPRO == "3" ~ "72",
                          ISCO2D %in% c("91", "95") & STAPRO == "3" ~ "73",
                          substr(ISCO2D, 0, 1) == "6" & STAPRO == "3" ~ "74",
                          
                          # Catégories par défaut pour les chômeurs et inactifs non classés
                          ILOSTAT == "3" & (MAINSTAT == "4" | AGE >= 65) ~ "Retired",
                          ILOSTAT == "2" | (ILOSTAT %in% c("3", "4") & AGE < 65) ~ "Inactive"),

        eseg2b = factor(if_else((is.na(eseg2) | eseg2 %in% c("Inactive", "Retired")) & !is.na(eseg),
                                paste0("8", as.numeric(eseg)), as.character(eseg2))),

        # ESeG en distinguant les agriculteurs
        esegb = factor(if_else(eseg2 == "41", "Farmers", as.character(eseg)),
                       levels=c("Managers", "Professionals", "Associate professionals",
                                "Farmers", "Small entrepreneurs",
                                "Skilled white-collar", "Skilled blue-collar", "Less skilled",
                                "Retired", "Inactive")),

        # ISCED 2011 disponible seulement à partir de 2014 (et ISCED 1997 trop mauvais au-delà de trois groupes)
        HAT11LEV = if_else(REFYEAR >= 2014, as.numeric(HAT11LEV), NA_real_),
        educ = factor(floor(HAT11LEV/100), levels=0:8,
                      labels=c("< Primary", "Primary", "Lower secondary",
                               "Upper secondary", "Post-secondary",
                               "Short tertiary", "Bachelor", "Master", "Doctorate")),
        educ1 = recode_factor(educ,
                              "< Primary"="≤ Primary", "Primary"="≤ Primary",
                              "Lower secondary"="Lower secondary",
                              "Upper secondary"="Upper secondary",
                              "Post-secondary"="Short tertiary", "Short tertiary"="Short tertiary",
                              "Master"="Tertiary ≥ Master", "Doctorate"="Tertiary ≥ Master"),
        educ2 = recode_factor(educ1,
                              "≤ Primary"="≤ Lower secondary", "Lower secondary"="≤ Lower secondary",
                              "Upper secondary"="Upper secondary",
                              "Short tertiary"="Tertiary ≤ Bachelor", "Bachelor"="Tertiary ≤ Bachelor",
                              "Tertiary ≥ Master"="Tertiary ≥ Master"),

        educ3 = recode_factor(HATLEV1D, L="≤ Lower secondary", M="Upper and post secondary", H="Tertiary")
    )

    ind <- droplevels(ind)

    name <- paste0(tolower(ind$COUNTRY[1]), ind$YEAR[1])
    indl[[name]] <- copy_to(lfs, ind, name=name, temporary=FALSE)
    rm(ind)
    gc()
}

## Base commune des individus

# Les noms posent problème (tables considérées comme arguments supplémentaires)
indiv1 <- compute(Reduce(union_all, indl[1:50]))
indiv1b <- compute(Reduce(union_all, indl[51:100]))
indiv2 <- compute(Reduce(union_all, indl[101:150]))
indiv3 <- compute(Reduce(union_all, indl[151:length(indl)]))
indiv <- Reduce(union_all, list(indiv1, indiv1b, indiv2, indiv4))

indiv <- compute(indiv, name="indiv", temporary=FALSE,
                 indexes=c("COUNTRY", "YEAR", "QHHNUM"))


### Couples
# Fusion directe avec ffdf
# Pays qui ont un identifiant de conjoint
# HHSPOU et HHLINK ne sont pas disponibles pour CH, DK, FI, IS, NO et SE dans les fichiers standard
# La doc p. 39 est incorrecte : les pays qui n'ont pas les données "détaillées" de ménage n'ont meme pas celles de base
# Pour FI et NO, on dispose de l'identifiant de ménage, mais on ne sait pas qui est le conjoint
couples <- inner_join(filter(indiv, SEX == "H" & HHSPOU > 0),
                      filter(indiv, SEX == "F" & HHSPOU > 0),
                      by=c("COUNTRY", "YEAR", "QHHNUM", "HHSEQNUM"="HHSPOU"),
                      suffix=c("H", "F")) %>%
           select(-c(COUNTRY2F, REGION2F, NUTS1F, NUTS1bF, NUTS2F, NUTS2bF, DEGURBAF)) %>%
           rename(COUNTRY2 = COUNTRY2H) %>% rename(REGION2 = REGION2H) %>%
           rename(NUTS1 = NUTS1H) %>% rename(NUTS1b = NUTS1bH) %>% rename(NUTS2 = NUTS2H) %>%
           rename(NUTS2b = NUTS2bH) %>% rename(DEGURBA = DEGURBAH) %>%
           # rename(COUNTRY2 = COUNTRY2H, REGION2 = REGION2H,
           #        NUTS1 = NUTS1H, NUTS1b = NUTS1bH, NUTS2 = NUTS2H, NUTS2b = NUTS2bH,
           #        DEGURBA = DEGURBAH) %>%
           mutate(weight = if_else(is.na(COEFFH), COEFFF, (COEFFH + COEFFF)/2)) %>%
           compute(name="couples", temporary = FALSE, indexes = c("COUNTRY", "NUTS1b", "NUTS2b", "YEAR"))


couples2 <- filter(couples, (AGEH >= 30 & AGEH < 60) & (AGEF >= 30 & AGEF < 60) & YEAR >= 2014)

## Vérifications

freqtable <- function(x, ..., wt = NULL, sort = FALSE, exclude = c(NA, NaN)) {
    vars <- lazyeval::lazy_dots(...)
    wt <- substitute(wt)
    counts <- count_(x, vars, wt = wt, sort = sort)
    xtabs(formula(paste(" n ~", paste(sapply(vars, "[", "expr"), collapse="+"))),
          counts, exclude=exclude, na.action=na.pass)
}

# Répartition des ménages selon leur nombre de couples
couples %>% count(YEAR, NUTS2b, QHHNUM) %>% count(n) %>%
    xtabs(formula=nn ~ NUTS2b + n + YEAR) %>% prop.table(c(1, 3)) %>% (function(x) round(x*1000))

# Proportion de valeurs manquantes
couples2 %>%
    mutate(missH = is.na(esegH) | esegH %in% c("Inactive", "Retired"),
           missF = is.na(esegF) | esegF %in% c("Inactive", "Retired")) %>%
    freqtable(COUNTRY, missH) %>%
    prop.table(1)

# Proportion d'individus en couple
x <- freqtable(filter(couples, (AGEH >= 30 & AGEH < 60) | (AGEF >= 30 & AGEF < 60)), YEAR, COUNTRY)
y <- filter(indiv, AGE >= 30 & AGE < 60) %>% freqtable(SEX, YEAR, COUNTRY)
1/sweep(y[,, colnames(x)], 2:3, x, "/")

# Taux d'activité des femmes
act <- (mutate(couples, inactF = esegF == "Inactive") %>%
            freqtable(NUTS2b, inactF) %>% prop.table(1))[, "FALSE"]
act

## Tables

# Poids des régions
wts <- indiv %>% filter(AGE >= 30 & AGE <= 60 & YEAR >= 2014) %>% freqtable(NUTS2b, wt=COEFF)
save(wts, file="weights NUTS2b 2014-2016.RData")

tab <- freqtable(couples2, esegH, esegF, NUTS2b)
tabw <- freqtable(couples2 , esegH, esegF, NUTS2b, wt=weight)
levs <- c("Managers", "Professionals", "Associate professionals", "Small entrepreneurs",
          "Skilled white-collar", "Skilled blue-collar", "Less skilled", "Inactive")
tab <- as.table(tab[levs, levs, ])
tabw <- as.table(tabw[levs, levs, ])
save(tab, tabw, file="tab NUTS2b eseg 2014-2016.RData")

# Couples en emploi
tab <- filter(couples2, ILOSTATH == "1" & ILOSTATF == "1") %>% freqtable(esegH, esegF, NUTS2b)
tabw <- filter(couples2, ILOSTATH == "1" & ILOSTATF == "1") %>% freqtable(esegH, esegF, NUTS2b, wt=weight)
levs <- c("Managers", "Professionals", "Associate professionals", "Small entrepreneurs",
          "Skilled white-collar", "Skilled blue-collar", "Less skilled")
tab <- as.table(tab[levs, levs, ])
tabw <- as.table(tabw[levs, levs, ])
save(tab, tabw, file="tab NUTS2b eseg employed 2014-2016.RData")

tab <- freqtable(couples2, educH, educF, NUTS2b)
tabw <- freqtable(couples2, educH, educF, NUTS2b, wt=weight)
levs <- c("< Primary", "Primary", "Lower secondary",
          "Upper secondary", "Post-secondary",
          "Short tertiary", "Bachelor", "Master", "Doctorate")
tab <- as.table(tab[levs, levs, ])
tabw <- as.table(tabw[levs, levs, ])
save(tab, tabw, file="tab NUTS2b educ 2014-2016.RData")

tab <- freqtable(couples2, educ2H, educ2F, NUTS2b)
tabw <- freqtable(couples2, educ2H, educ2F, NUTS2b, wt=weight)
levs <- c("≤ Lower secondary", "Upper secondary", "Tertiary ≤ Bachelor", "Tertiary ≥ Master")
tab <- as.table(tab[levs, levs, ])
tabw <- as.table(tabw[levs, levs, ])
save(tab, tabw, file="tab NUTS2b educ2 2014-2016.RData")

tab <- freqtable(couples2, educ3H, educ3F, NUTS2b)
tabw <- freqtable(couples2, educ3H, educ3F, NUTS2b, wt=weight)
levs <- c("≤ Lower secondary", "Upper and post secondary", "Tertiary")
tab <- as.table(tab[levs, levs, ])
tabw <- as.table(tabw[levs, levs, ])
save(tab, tabw, file="tab NUTS2b educ3 2014-2016.RData")
